@@ -111,6 +111,11 @@ def final(fn):
     return _wrapped
 
 
+_RFM_TEST_KIND_MIXIN = 0
+_RFM_TEST_KIND_COMPILE = 1
+_RFM_TEST_KIND_RUN = 2
+
+
 class RegressionMixin(metaclass=RegressionTestMeta):
     '''Base mixin class for regression tests.
 
@@ -124,17 +129,7 @@ class RegressionMixin(metaclass=RegressionTestMeta):
     .. versionadded:: 3.4.2
     '''
 
-    def __getattr__(self, name):
-        ''' Intercept the AttributeError if the name is a required variable.'''
-        if (name in self._rfm_var_space and
-            not self._rfm_var_space[name].is_defined()):
-            raise AttributeError(
-                f'required variable {name!r} has not been set'
-            ) from None
-        else:
-            raise AttributeError(
-                f'{type(self).__qualname__} object has no attribute {name!r}'
-            )
+    _rfm_regression_class_kind = _RFM_TEST_KIND_MIXIN
 
 
 class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
@@ -189,6 +184,8 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
     '''
 
+    _rfm_regression_class_kind = _RFM_TEST_KIND_COMPILE | _RFM_TEST_KIND_RUN
+
     def disable_hook(self, hook_name):
         '''Disable pipeline hook by name.
 
@@ -856,7 +853,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         # injected after __new__ has returned, so we schedule this function
         # call as a pre-init hook).
         obj.__deferred_rfm_init = obj.__rfm_init__(*args,
-                                                   name=cls.__qualname__,
                                                    prefix=prefix, **kwargs)
 
         # Build pipeline hook registry and add the pre-init hook
@@ -901,12 +897,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             )
 
     @deferrable
-    def __rfm_init__(self, *args, name=None, prefix=None, **kwargs):
-        if name is not None:
-            self.name = name
-
-            # Add the parameters to the name.
-            self.name += self._append_parameters_to_name()
+    def __rfm_init__(self, *args, prefix=None, **kwargs):
+        if not hasattr(self, 'name'):
+            self.name = type(self).fullname(self.variant_num)
 
             # Add the parameters from the parameterized_test decorator.
             if args or kwargs:
@@ -966,13 +959,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         # Disabled hooks
         self._disabled_hooks = set()
 
-    def _append_parameters_to_name(self):
-        if self._rfm_param_space.params:
-            return '_' + '_'.join([util.toalphanum(str(self.__dict__[key]))
-                                   for key in self._rfm_param_space.params])
-        else:
-            return ''
-
     @classmethod
     def _process_hook_registry(cls):
         '''Process and validate the pipeline hooks.'''
@@ -1017,6 +1003,24 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         return super().__getattribute__(name)
 
+    def __getattr__(self, name):
+        ''' Intercept the special builtin-related AttributeError.'''
+
+        if (name in self._rfm_var_space and
+            not self._rfm_var_space[name].is_defined()):
+            raise AttributeError(
+                f'required variable {name!r} has not been set'
+            ) from None
+        elif name in self._rfm_fixture_space:
+            raise AttributeError(
+                f'fixture {name!r} has not yet been resolved: '
+                f'fixtures are resolved during the setup stage'
+            )
+        else:
+            raise AttributeError(
+                f'{type(self).__qualname__!r} object has no attribute {name!r}'
+            )
+
     # Export read-only views to interesting fields
 
     @property
@@ -1050,6 +1054,41 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         '''
         return rt.runtime().system
 
+    @property
+    def variant_num(self):
+        '''The variant number of the test.
+
+        This number should be treated as a unique ID representing a unique
+        combination of the available parameter and fixture variants.
+
+        :type: :class:`int`
+        '''
+        return getattr(self, '_rfm_variant_num', None)
+
+    @property
+    def param_variant(self):
+        '''The point in the parameter space for the test.
+
+        This can be seen as an index to the paraemter space representing a
+        unique combination of the parameter values. This number is directly
+        mapped from ``variant_num``.
+
+        :type: :class:`int`
+        '''
+        return getattr(self, '_rfm_param_variant', None)
+
+    @property
+    def fixture_variant(self):
+        '''The point in the fixture space for the test.
+
+        This can be seen as an index to the fixture space representing a
+        unique combination of the fixture variants. This number is directly
+        mapped from ``variant_num``.
+
+        :type: :class:`int`
+        '''
+        return getattr(self, '_rfm_fixt_variant', None)
+
     @property
     def perfvalues(self):
         return util.MappingView(self._perfvalues)
@@ -1204,6 +1243,90 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         return self.local or self._current_partition.scheduler.is_local
 
+    def is_fixture(self):
+        '''Check if the test is a fixture.'''
+        return getattr(self, '_rfm_is_fixture', False)
+
+    def _resolve_fixtures(self):
+        '''Resolve the fixture dependencies and inject the fixture handle.
+
+        The fixture handle will point directly to the fixture object when the
+        associated fixture uses a 'fork' action. However, when a fixture uses
+        the 'join' action, the injected handle will point to a list with all
+        the available fixture variants.
+        '''
+
+        current_part = self.current_partition.fullname
+        current_env = self.current_environ.name
+
+        # Get the declared and registered fixtures
+        fixtures = type(self)._rfm_fixture_space
+        registry = getattr(self, '_rfm_fixture_registry', None)
+
+        # If this instance does not have a variant number, return
+        if self.fixture_variant is None:
+            return
+
+        # Get the fixture variants required for this test variant.
+        # This would retrieve the same information than when calling
+        # ``type(self).get_variant_info(self.variant_num)['fixtures']``.
+        target_fixt_variants = type(self).fixture_space[self.fixture_variant]
+
+        for handle_name, f in fixtures.items():
+            # Get the target variants for this fixture
+            target_variants = target_fixt_variants[handle_name]
+
+            # Prepare the getdep argumens based on the fixture's scope
+            if f.scope == 'session':
+                part = '*'
+                environ = '*'
+            elif f.scope == 'partition':
+                part = None
+                environ = '*'
+            else:
+                part = None
+                environ = None
+
+            # List to store all the targeted fixture variants
+            deps = []
+
+            # Scan the fixture registry and resolve the fixtures.
+            # NOTE: The fixture registry can have multiple fixture instances
+            # registered under the same fixture class. So the loop below must
+            # also inspect the fixture data the instance was registered with.
+            for fixt_name, fixt_data in registry[f.cls].items():
+                if f.scope != fixt_data.scope:
+                    continue
+                elif fixt_data.variant_num not in target_variants:
+                    continue
+                elif f.scope == 'partition':
+                    if fixt_data.partitions[0] != current_part:
+                        continue
+                elif f.scope == 'environment':
+                    if (fixt_data.environments[0] != current_env or
+                        fixt_data.partitions[0] != current_part):
+                        continue
+
+                # Resolve the fixture
+                deps.append(self.getdep(fixt_name, environ, part))
+
+            if f.action == 'fork':
+                # When using the fork action, a fixture handle can only have
+                # a single fixture instance attached. This could only happen
+                # if either any of the above ifs is buggy or if a fixture with
+                # a fork action ever has more than one index per fork variant.
+                # None of this can happen from user input, but this must stay
+                # here to ensure the unit tests do not fail silently.
+                if len(deps) > 1:
+                    raise PipelineError(
+                        f'fixture {handle_name!r} has more than one instances'
+                    )
+
+                deps = deps[0]
+
+            # Inject the fixtures
+            setattr(self, handle_name, deps)
+
     def _setup_paths(self):
         '''Setup the check's dynamic paths.'''
         self.logger.debug('Setting up test paths')
@@ -1275,6 +1398,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         self._current_partition = partition
         self._current_environ = environ
         self._setup_paths()
+        self._resolve_fixtures()
         self._job = self._setup_job(f'rfm_{self.name}_job',
                                     self.local,
                                     **job_opts)
@@ -1294,7 +1418,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
     def _clone_to_stagedir(self, url):
         self.logger.debug(f'Cloning URL {url} into stage directory')
-        osext.git_clone(self.sourcesdir, self._stagedir)
+        osext.git_clone(
+            self.sourcesdir, self._stagedir,
+            # FIXME: cast to float explicitly due to GH #2246
+            timeout=float(rt.runtime().get_option('general/0/git_timeout'))
+        )
 
     @final
     def compile(self):
@@ -2026,6 +2154,10 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
         .. versionadded:: 2.21
 
+        .. versionchanged:: 3.8.0
+           Setting ``environ`` or ``part`` to ``'*'`` will skip the match
+           check on the environment and partition, respectively.
+
         '''
         if self.current_environ is None:
             raise DependencyError(
@@ -2042,9 +2174,10 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             raise DependencyError('no test case is associated with this test')
 
         for d in self._case().deps:
-            if (d.check.name == target and
-                d.environ.name == environ and
-                d.partition.name == part):
+            mask = int(d.check.name == target)
+            mask |= (int(d.partition.name == part) | int(part == '*')) << 1
+            mask |= (int(d.environ.name == environ) | int(environ == '*')) << 2
+            if mask == 7:
                 return d.check
 
         raise DependencyError(f'could not resolve dependency to ({target!r}, '
@@ -2095,6 +2228,8 @@ class RunOnlyRegressionTest(RegressionTest, special=True):
     module.
     '''
 
+    _rfm_regression_class_kind = _RFM_TEST_KIND_RUN
+
     def setup(self, partition, environ, **job_opts):
         '''The setup stage of the regression test pipeline.
 
@@ -2107,6 +2242,7 @@ class RunOnlyRegressionTest(RegressionTest, special=True):
         self._job = self._setup_job(f'rfm_{self.name}_job',
                                     self.local,
                                     **job_opts)
+        self._resolve_fixtures()
 
     def compile(self):
         '''The compilation phase of the regression test pipeline.
@@ -2149,6 +2285,8 @@ class CompileOnlyRegressionTest(RegressionTest, special=True):
     module.
     '''
 
+    _rfm_regression_class_kind = _RFM_TEST_KIND_COMPILE
+
     def setup(self, partition, environ, **job_opts):
         '''The setup stage of the regression test pipeline.
 
@@ -2162,6 +2300,7 @@ class CompileOnlyRegressionTest(RegressionTest, special=True):
         self._build_job = self._setup_job(f'rfm_{self.name}_build',
                                           self.local or self.build_locally,
                                           **job_opts)
+        self._resolve_fixtures()
 
     @property
     @deferrable
